﻿using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using Autofac;
using BPA.Component.ApolloClient;
using BPA.Component.CAP;
using BPA.Component.Extensions;
using BPA.Component.LogClient.Extensions;
using BPA.Component.SDKCommon;
using BPA.Component.WebApiExtensions.Extensions;
using BPA.SaaS.Annex.Api.Model;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Diagnostics.HealthChecks;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.OpenApi.Models;
using IPAddress = System.Net.IPAddress;

namespace BPA.SaaS.Annex.Api.Bootstrap;

/// <summary>
/// 启动配置
/// </summary>
public static class SetupBootstrap
{
    private static IConfiguration _configuration;

    /// <summary>
    /// 添加配置中心
    /// </summary>
    /// <param name="configuration"></param>
    public static IConfiguration SetupConfiguration(this IConfiguration configuration)
    {
        _configuration = configuration;

        // 初始化化 apollo配置
        return configuration.AddApolloConfiguration();
    }

    /// <summary>
    /// DI Inject
    /// </summary>
    public static IServiceCollection AddBPAMiddlewareServices(this IServiceCollection services, string[] args)
    {
        if (services == null) throw new ArgumentNullException(nameof(services));

        services.AddApollo<AnnexApiConfig>();
        services.AddBPALog();
        services.AddRedis();
        services.AddRabbitMQ();
        services.AddTransient(typeof(Lazy<>), typeof(BPADefaultLazy<>));
        services.AddHttpContextAccessor();
        services.AddControllers(options => { options.AddDefaultMvcOptions(); });
        services.AddEndpointsApiExplorer();
        services.AddServiceSDK(_configuration);
        services.AddHealthChecks();

        services.AddSwaggerGen(options =>
        {
            options.SwaggerDoc("v1", new OpenApiInfo
            {
                Title = AppDomain.CurrentDomain.FriendlyName,
                Description = AppDomain.CurrentDomain.FriendlyName,
                Version = "v1"
            });
            options.CustomSchemaIds(type => type.GetDescribe(true, "BPA.", "BPA.SaaS."));
        });

        return services;
    }

    /// <summary>
    /// DI Register
    /// </summary>
    public static ContainerBuilder SetupConfigureContainer(this ContainerBuilder builder)
    {
        // // 注入DB
        // builder.RegisterType<BasicDbSugarClient>().InstancePerLifetimeScope();
        //
        // // 服务层
        // builder.RegisterAssemblyTypes(typeof(HomeService).Assembly)
        //     .Where(t => t.Name.EndsWith("Service"))
        //     .AsImplementedInterfaces();
        //
        // // 仓储层
        // builder.RegisterAssemblyTypes(typeof(HomeRepository).Assembly)
        //     .Where(t => t.Name.EndsWith("Repository"))
        //     .AsImplementedInterfaces();

        return builder;
    }

    /// <summary>
    /// pre run
    /// </summary>
    /// <param name="serviceProvider"></param>
    public static void PreHeatRun(this IServiceProvider serviceProvider)
    {
        serviceProvider.GetService<AnnexApiConfig>();
        serviceProvider.PreHeatRunRedis();
        serviceProvider.PreHeatRunRabbitMQ();
        serviceProvider.SetDebug(true);
        serviceProvider.GetService<ICapPublisher>();
    }

    /// <summary>
    /// APP健康检查
    /// </summary>
    /// <param name="app"></param>
    public static void UseBpaHealthChecks(this IApplicationBuilder app)
    {
        var ips = new List<dynamic>();
        foreach (var network in NetworkInterface.GetAllNetworkInterfaces())
        {
            var ipAddress = network.GetIPProperties()
                .UnicastAddresses
                .FirstOrDefault(p => p.Address.AddressFamily == AddressFamily.InterNetwork && !IPAddress.IsLoopback(p.Address))
                ?.Address.ToString();
            if (ipAddress == null)
                continue;
            ips.Add(new {name = network.Name, ip = ipAddress});
        }

        var ipsJson = ips.ToJson();

        app.UseHealthChecks("/health", new HealthCheckOptions
        {
            ResponseWriter = (httpContext, _) => httpContext.Response.WriteAsync(ipsJson, Encoding.UTF8)
        });
    }
}